function [ communitiesPeople, mutualInfo ] = DetectCommunitiesBipartite( testData, numLeftSet, communityLabels, displayFlag)
%Performs community detection on a bipartite network
%   testData: adjacency matrix
%   numLeftSet: number of nodes in the left side of the bipartite network
%   communityLabels: the actual community labels to compare against
%   displayFlag: 0 displays nothing, 1,2,3 display various attributes


    %Initialize the members to all initially be in a distinct community
    communitiesPeople = 1:(numLeftSet);
    communitiesPeople = communitiesPeople';
    communitiesPapers = 1:(length(testData) - numLeftSet);
    communitiesPapers = communitiesPapers';
    
    if displayFlag == 1
        %%
        DisplayGraph(testData, numLeftSet, communitiesPeople, communitiesPapers);
    end
    
    n = size(testData,1);
    
    %Generate the similarity scores using the Erdos numbers
    weightedMat = GenErdosNumFast(testData);            %This uses the GENs
%     weightedMat = MarkovTransitionSimScore(testData);  %This uses a
    %different metric based on markov transition probabilities
%     weightedMat = testData;
    
    %Used to detect convergence of the algorithm
    communitiesPeopleOld = communitiesPeople - 1;
    
    numIterations = 0;
    i = 1; % counter for combining communities
    maxNumIterations = 100; %100
    
    if displayFlag == 2
        %%
        numRightSet = length(testData) - numLeftSet;
        subplot(2,2,1);
        surf(weightedMat(1:numLeftSet, numLeftSet+1:end), 'EdgeColor', 'None');
        axis([1 numLeftSet 1 numRightSet]);
        
        subplot(2,2,2);
        surf(weightedMat(1:numLeftSet, 1:numLeftSet), 'EdgeColor', 'None');
        axis([1 numLeftSet 1 numLeftSet]);
        
        subplot(2,2,3);
        surf(weightedMat(numLeftSet+1:end, numLeftSet+1:end), 'EdgeColor', 'None');
        axis([1 numRightSet 1 numRightSet]);
        
        subplot(2,2,4);
        spy(testData);
    end
    %%
    %Iterative Refinement algorithm loops until it converges or reaches the
    %maximum number of iterations
    while ~min(communitiesPeopleOld == communitiesPeople) && (numIterations < maxNumIterations)
        %%
        communitiesPeopleOld = communitiesPeople;

        for j = 1:3
            [ communitiesPeople, communitiesPapers] = IterateWithinSide(numLeftSet, communitiesPeople, communitiesPapers, weightedMat);
        end
%%
        if displayFlag == 1
            %%
            figure
            DisplayGraph(testData, numLeftSet, communitiesPeople, communitiesPapers);
            drawnow;
        end
        %%
        for j = 1:3
            [ communitiesPeople, communitiesPapers] = IterateBetweenSides(numLeftSet, communitiesPeople, communitiesPapers, weightedMat);
        end

        if displayFlag == 1
            %%
            figure
            DisplayGraph(testData, numLeftSet, communitiesPeople, communitiesPapers);
            drawnow;
        end
%%
        if mod(i, 5) == 0 || min(communitiesPeopleOld == communitiesPeople)
            %%
            [ communitiesPeople, communitiesPapers] = CombineCommunities(weightedMat, numLeftSet, communitiesPeople, communitiesPapers);
            i = 0;
        end
                
        if displayFlag == 1
            %%
            figure
            DisplayGraph(testData, numLeftSet, communitiesPeople, communitiesPapers);
            drawnow;
        end
        i = i + 1;
        numIterations = numIterations + 1;
    end
    
    if displayFlag == 3        
%         subplot(2,1,1);
%         figure;
%         surf(weightedMat(1:numLeftSet, 1:numLeftSet), 'EdgeColor', 'None');
%         axis([1 numLeftSet 1 numLeftSet]);
        
%         subplot(2,1,2);
        figure;
        plot(communitiesPeople,'o');
        axis([1 length(communitiesPeople) min(communitiesPeople)-1 max(communitiesPeople)+1])
    end
    
    %Computes the mutual information between the algorithms predicted
    %communities and the actual communities
    mutualInfo = - 1;
    if ~isempty(communityLabels)
        mutualInfo = MutualInformation(communitiesPeople, communityLabels);
    end
end