function [ erdosNumNew] = GenErdosNumFast( adjMatrix )
%Computes the generalized erdos numbers for a matrix
    eyeN = eye(length(adjMatrix)) == 1;
    erdosNum = zeros(length(adjMatrix));
    erdosNumNew = ones(length(adjMatrix)) - eye(length(adjMatrix));
    
    nodeWeights = sum(adjMatrix, 2);
    tol = 5*10^(-3);
    numIterations = 0;
    maxIterations = 200;
    
    maxDiff = abs(max(max(erdosNumNew - erdosNum)));
    
    %Identifies non connected components in the graph
    katzMask = WeightedKatzMask(adjMatrix./nodeWeights(:, ones(1, length(nodeWeights))),.1);
    notConnectedComp = katzMask == 0;
    
    ind = adjMatrix > 0;
    adjMatrixRecip = 1./adjMatrix;
    val = zeros(length(adjMatrix));
    
    while(maxDiff > tol && numIterations < maxIterations)
        erdosNum = erdosNumNew;
        
        for i = 1:length(adjMatrix)
            tmp = erdosNum(i,:);
            erdosNumI = tmp(ones(length(tmp), 1), :);
            val = adjMatrix./(erdosNumI + adjMatrixRecip);
            erdosNumNew(i, :) = nodeWeights./sum(val,2);
        end
        
        erdosNumNew(eyeN) = 0;
        erdosNumNew(notConnectedComp) = 0;
        maxDiff = max(max(abs(erdosNumNew - erdosNum)));
        numIterations = numIterations + 1;
        
    end
    

    maxVal = max(max(erdosNumNew));

    %Added this postprocessing step (switches to similarity score)
%     maxWeight = repmat(max(erdosNumNew,[],2),1,length(erdosNumNew)); 
%     erdosNumNew = erdosNumNew./maxWeight;
%     maxVal = max(max(erdosNumNew));
    
    erdosNumNew(eyeN) = maxVal;
    erdosNumNew(notConnectedComp) = maxVal;
    erdosNumNew = erdosNumNew/maxVal;
    erdosNumNew = 1 - erdosNumNew;
end