function [ communitiesPeople, communitiesPapers ] = IterateBetweenSides( numLeftSet, communitiesPeople, communitiesPapers, weightedMat)
%This part of the algorithm iterates between the two sides of the bipartite
%network

    n = size(weightedMat,1);
    
    %Compute normalized value of closeness to each community on the other
    %side of the bipartite network
    peopleMatrix = double(repmat(communitiesPeople,1,numLeftSet) == repmat(communitiesPeople',numLeftSet,1));
    peopleMatrix = peopleMatrix./repmat(sum(peopleMatrix,2),1,numLeftSet);
    
    connectionsBetween = weightedMat(1:numLeftSet, (numLeftSet + 1):end);
    connectionsBetweenRev = weightedMat((numLeftSet + 1):end, 1:numLeftSet);
        
    refinementPapers = connectionsBetweenRev*peopleMatrix;
    communitiesPapersNew = communitiesPapers;
    
    papersMatrix = double(repmat(communitiesPapers,1,n-numLeftSet) == repmat(communitiesPapers',n-numLeftSet,1));
    papersMatrix = papersMatrix./repmat(sum(papersMatrix,2),1,n-numLeftSet);
    refinementPeople = connectionsBetween*papersMatrix;
    
    %Iterative Refinement Phase for Papers
    for j = 1:(n-numLeftSet)
        communitiesPapersNew(j) = communitiesPeople(find(refinementPapers(j,:) == max(refinementPapers(j, :)),1));        
    end
    
    %Iterative Refinement Phase for People
    for i = 1:numLeftSet
        communitiesPeople(i) = communitiesPapers(find(refinementPeople(i,:) == max(refinementPeople(i,:)),1));
    end
    
    %Reset communitiesPapers after the People side of the bipartite graph
    %has been updated
    communitiesPapers = communitiesPapersNew;
end