function [ communitiesPeople, communitiesPapers ] = IterateWithinSide( numLeftSet, communitiesPeople, communitiesPapers, weightedMat )
%This part of the algorithm iterates within a given side of the Bipartite
%Network

    n = size(weightedMat, 1);
    
    %Computes the weighted closeness that each node feels to every other
    %node on the same side of the bipartite graph
    peopleGraph = weightedMat(1:numLeftSet, 1:numLeftSet);
    paperGraph = weightedMat(numLeftSet+1:end, numLeftSet + 1:end);    
    
    peopleMatrix = double(repmat(communitiesPeople,1,numLeftSet) == repmat(communitiesPeople',numLeftSet,1));
    peopleMatrix = peopleMatrix./repmat(sum(peopleMatrix,2),1,numLeftSet);
    
    papersMatrix = double(repmat(communitiesPapers,1,n-numLeftSet) == repmat(communitiesPapers',n-numLeftSet,1));
    papersMatrix = papersMatrix./repmat(sum(papersMatrix,2),1,n-numLeftSet);

    weightPaperCommunity = paperGraph * papersMatrix;
    weightPeopleCommunity = peopleGraph * peopleMatrix;

    %Iterate within the set and compute the closest community for Paper
    %side
    for j = 1:(n-numLeftSet)
        simScoreCommunity = max(weightPaperCommunity(j, :))/weightPaperCommunity(j,j);
        
        %Only change community if closer to the new value than itself
        if simScoreCommunity > 1
            communitiesPapers(j) = communitiesPapers(find(weightPaperCommunity(j,:) == max(weightPaperCommunity(j, :)),1));
        end
    end

    %Iterate within the set and compute the closest community for People
    %side
    for i = 1:numLeftSet
        simScoreCommunity = max(weightPeopleCommunity(i, :))/weightPeopleCommunity(i,i);
        
        %Only change community if closer to the new value than itself
        if simScoreCommunity > 1
            communitiesPeople(i) = communitiesPeople(find(weightPeopleCommunity(i,:) == max(weightPeopleCommunity(i,:)),1));
        end
    end
    
end