function [map,map_mu] = QCiteration(v,f,point,target,epsilon)
% QC Iteration algorithm. 
%
% The codes in this subfolder are from the following paper:
% L. M. Lui, K. C. Lam, S. T. Yau, X. Gu, 
% Teichmuller mapping (T-map) and its applications to landmark matching registration.
% SIAM Journal on Imaging Sciences 7 (1), 391-426, 2014

mu = zeros(length(f),1);
TR = TriRep(f',v(1,:)',v(2,:)');
ring = vertexAttachments(TR);

constraint_x = [point,target(:,1)];
constraint_y = [point,target(:,2)];

if ~exist('epsilon','var')
    epsilon = .01; 
end

map = map_reconstruct(mu,v,f,ring,constraint_x,constraint_y,0);
update_mu = BC(v,f,map);
mu_std = 1;
count = 1;
numofiter = 0;

while (mu_std > epsilon) || (count > 0) 
    update_mu = mu_chop(update_mu,1,0);
    Smooth_mu = mean(abs(update_mu)).*(cos(angle(real(update_mu)+sqrt(-1)*imag(update_mu)))+...
        sqrt(-1)*sin(angle(real(update_mu)+sqrt(-1)*imag(update_mu))));
    map = map_reconstruct(Smooth_mu,v,f,ring,constraint_x,constraint_y,0);
    update_mu = BC(v,f,map);
    mu_std = std(abs(update_mu));
    count = length(find(abs(update_mu)>=1));
    numofiter = numofiter + 1;
    if numofiter > 500
        break;
    end
end

map_mu = BC(v,f,map);
end
